/*
 * @(#)ProducerUtil.java	1.4 02/09/06
 *
 * Copyright 2005 Sun Microsystems, Inc. All Rights Reserved
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms. 
 *
 */

package com.sun.messaging.jmq.jmsserver.management.util;

import java.util.Iterator;

import javax.management.openmbean.CompositeData;
import javax.management.openmbean.CompositeDataSupport;
import javax.management.openmbean.CompositeType;
import javax.management.openmbean.OpenType;
import javax.management.openmbean.SimpleType;
import javax.management.openmbean.OpenDataException;

import com.sun.messaging.jms.management.server.*;

import com.sun.messaging.jmq.jmsserver.Globals;
import com.sun.messaging.jmq.jmsserver.core.ProducerUID;
import com.sun.messaging.jmq.jmsserver.core.Producer;
import com.sun.messaging.jmq.jmsserver.core.Destination;
import com.sun.messaging.jmq.jmsserver.core.DestinationUID;
import com.sun.messaging.jmq.jmsserver.service.ConnectionUID;
import com.sun.messaging.jmq.jmsserver.resources.BrokerResources;
import com.sun.messaging.jmq.util.admin.ConnectionInfo;
import com.sun.messaging.jmq.net.IPAddress;

import com.sun.messaging.jmq.jmsserver.util.BrokerException;

public class ProducerUtil {
    /*
     * Producer Info item names for Monitor MBeans
     */
    private static final String[] producerInfoMonitorItemNames = {
                            ProducerInfo.CONNECTION_ID,
                            ProducerInfo.DESTINATION_NAME,
                            ProducerInfo.DESTINATION_TYPE,
                            ProducerInfo.FLOW_PAUSED,
                            ProducerInfo.HOST,
                            ProducerInfo.NUM_MSGS,
                            ProducerInfo.PRODUCER_ID,
                            ProducerInfo.SERVICE_NAME,
                            ProducerInfo.USER
                    };

    /*
     * Producer Info item description for Monitor MBeans
     * TBD: use real descriptions
     */
    private static final String[] producerInfoMonitorItemDesc = producerInfoMonitorItemNames;

    /*
     * Producer Info item types for Monitor MBeans
     */
    private static final OpenType[] producerInfoMonitorItemTypes = {
			    SimpleType.STRING,		// connection ID
			    SimpleType.STRING,		// dest name
			    SimpleType.STRING,		// dest type
			    SimpleType.BOOLEAN,		// flow paused
			    SimpleType.STRING,		// host
			    SimpleType.LONG,		// num msgs
			    SimpleType.STRING,		// producer ID
			    SimpleType.STRING,		// service name
			    SimpleType.STRING		// user
                    };

    /*
     * Producer Info composite type for Monitor MBeans
     */
    private static CompositeType monitorCompType = null;

    public static ConnectionUID getConnectionUID(ProducerUID pid)  {
	Producer p = Producer.getProducer(pid);
	ConnectionUID cxnId;

	if (p == null)  {
	    return (null);
	}

	cxnId = p.getConnectionUID();

	return (cxnId);
    }


    /*
    public static Destination getDestination(ProducerUID pid)  {
	Producer p = Producer.getProducer(pid);
	DestinationUID did = p.getDestinationUID();
        Destination d = Destination.getDestination(did);

	return (d);
    }
    */

    public static String[] getProducerIDs()  {
	int numProducers = Producer.getNumProducers();
	String ids[];
	Iterator producers;

	if (numProducers <= 0)  {
	    return (null);
	}

	ids = new String [ numProducers ];

	producers = Producer.getAllProducers();
	int i = 0;
	while (producers.hasNext()) {
	    Producer oneProd = (Producer)producers.next();
	    long prodID = oneProd.getProducerUID().longValue();

	    ids[i] = Long.toString(prodID);
	    i++;
	}

	return (ids);
    }

    public static CompositeData[] getProducerInfo()
				throws BrokerException, OpenDataException  {
	String[] ids = getProducerIDs();

	if (ids == null)  {
	    return (null);
	}

	CompositeData cds[] = new CompositeData [ ids.length ];

	for (int i = 0; i < ids.length; ++i)  {
	    cds[i] = getProducerInfo(ids[i]);
	}
	
	return (cds);
    }

    public static CompositeData getProducerInfo(String producerID) 
				throws BrokerException, OpenDataException  {
	CompositeData cd = null;
	ProducerUID pid = null;
        BrokerResources	rb = Globals.getBrokerResources();

	if (producerID == null)  {
	    throw new 
		IllegalArgumentException(rb.getString(rb.X_JMX_NULL_PRODUCER_ID_SPEC));
	}

	long longPid = 0;

	try  {
	    longPid = Long.parseLong(producerID);
	} catch (NumberFormatException e)  {
	    throw new 
		IllegalArgumentException(rb.getString(rb.X_JMX_INVALID_PRODUCER_ID_SPEC, producerID));
	}

	pid = new ProducerUID(longPid);

	if (pid == null)  {
	    throw new BrokerException(rb.getString(rb.X_JMX_PRODUCER_NOT_FOUND, producerID));
	}

	cd = getProducerInfo(pid);

	return (cd);
    }


    public static String getConnectionID(ProducerUID pid)  {
        ConnectionUID cxnId = getConnectionUID(pid);

	if (cxnId == null)  {
	    return (null);
	}

	return(Long.toString(cxnId.longValue()));
    }

    public static String getDestinationName(ProducerUID pid)  {
	Producer p = Producer.getProducer(pid);

	if (p != null)  {
	    DestinationUID did = p.getDestinationUID();
	    Destination d = Destination.getDestination(p.getDestinationUID());

	    if (d == null)  {
		return (null);
	    }

	    return(d.getDestinationName());
	}

        return (null);
    }

    public static String getDestinationType(ProducerUID pid)  {
	Producer p = Producer.getProducer(pid);

	if (p != null)  {
	    DestinationUID did = p.getDestinationUID();
	    Destination d = Destination.getDestination(p.getDestinationUID());

	    if (d == null)  {
		return (null);
	    }

	    return(d.isQueue() ? DestinationType.QUEUE : DestinationType.TOPIC);
	}

	return (null);
    }

    public static Boolean getFlowPaused(ProducerUID pid)  {
	Producer p = Producer.getProducer(pid);

	if (p == null)  {
	    return (null);
	}

        return (new Boolean(p.isPaused()));
    }

    public static String getHost(ProducerUID pid)  {
	Producer p = Producer.getProducer(pid);
	ConnectionUID cxnId = null;

	if (p == null)  {
	    return (null);
	}

	cxnId = p.getConnectionUID();

	if (cxnId == null)  {
	    return (null);
	}

	ConnectionInfo cxnInfo = ConnectionUtil.getConnectionInfo(cxnId.longValue());

	if (cxnInfo == null)  {
	    return (null);
	}

	String host = null;

	if (cxnInfo.remoteIP != null) {
            host = String.valueOf(
		IPAddress.rawIPToString(cxnInfo.remoteIP, true, true));
        }

	return (host);
    }

    public static Long getNumMsgs(ProducerUID pid)  {
	Producer p = Producer.getProducer(pid);

	if (p == null)  {
	    return (null);
	}

        return (new Long(p.getMsgCnt()));
    }

    public static String getServiceName(ProducerUID pid)  {
	Producer p = Producer.getProducer(pid);
	ConnectionUID cxnId = null;

	if (p == null)  {
	    return (null);
	}

	cxnId = p.getConnectionUID();

	if (cxnId == null)  {
	    return (null);
	}

	return(ConnectionUtil.getServiceOfConnection(cxnId.longValue()));
    }

    public static String getUser(ProducerUID pid)  {
	Producer p = Producer.getProducer(pid);
	ConnectionUID cxnId = null;

	if (p == null)  {
	    return (null);
	}

	cxnId = p.getConnectionUID();

	if (cxnId == null)  {
	    return (null);
	}

	ConnectionInfo cxnInfo = ConnectionUtil.getConnectionInfo(cxnId.longValue());

	return (cxnInfo.user);
    }

    private static CompositeData getProducerInfo(ProducerUID pid) 
						throws OpenDataException  {
	Object[] producerInfoMonitorItemValues = {
                            getConnectionID(pid),
                            getDestinationName(pid),
                            getDestinationType(pid),
                            getFlowPaused(pid),
                            getHost(pid),
                            getNumMsgs(pid),
			    Long.toString(pid.longValue()),
                            getServiceName(pid),
                            getUser(pid)
			};
	CompositeData cd = null;

        if (monitorCompType == null)  {
            monitorCompType = new CompositeType("ProducerMonitorInfo", "ProducerMonitorInfo", 
                        producerInfoMonitorItemNames, producerInfoMonitorItemDesc, 
				producerInfoMonitorItemTypes);
        }

	cd = new CompositeDataSupport(monitorCompType, 
			producerInfoMonitorItemNames, producerInfoMonitorItemValues);
	
	return (cd);
    }

}
