/*
 * @(#)ConnectionFactoryAdapter.java	1.10 06/02/09
 *
 * Copyright 2005 Sun Microsystems, Inc. All Rights Reserved.
 * SUN PROPRIETARY/CONFIDENTIAL.
 * Use is subject to license terms.
 *
 */
 
package com.sun.messaging.jms.ra;

import javax.jms.*; 
import javax.resource.*;
import javax.resource.spi.*;
import javax.naming.Reference;
import javax.naming.Referenceable;
import javax.naming.NamingException;

import java.util.logging.Logger;

import javax.naming.Reference;
import javax.naming.Referenceable;
import javax.naming.NamingException;

import com.sun.messaging.jmq.jmsclient.XAConnectionImpl; 
 
/**
 * Implements the JMS ConnectionFactory interface for the S1 MQ RA.
 * An instance of this class is returned to the application server
 * when it uses the createConnectionFactory method of the
 * ManagedConnectionFactory.
 *
 * @author George Tharakan
 */
 
public class ConnectionFactoryAdapter
implements javax.jms.ConnectionFactory,
           javax.jms.QueueConnectionFactory,
           javax.jms.TopicConnectionFactory,
           javax.resource.Referenceable,
           java.io.Serializable
{
    /** The ManagedConnectionFactory instance that created this instance */
    private com.sun.messaging.jms.ra.ManagedConnectionFactory mcf = null;

    /** The ConnectionManager instance */
    private javax.resource.spi.ConnectionManager cm = null;

    /** The Reference instance */
    private Reference reference = null;

    /* Loggers */
    private static transient final String _className = "com.sun.messaging.jms.ra.ConnectionFactoryAdapter";
    protected static transient Logger _loggerJF;
    protected static transient Logger _loggerOC;
    protected static transient final String _lgrNameOutboundConnection = "javax.resourceadapter.mqjmsra.outbound.connection";
    protected static transient final String _lgrNameJMSConnectionFactory = "javax.jms.ConnectionFactory.mqjmsra";
    protected static transient final String _lgrMIDPrefix = "MQJMSRA_FA";
    protected static transient final String _lgrMID_EET = _lgrMIDPrefix + "1001: ";
    protected static transient final String _lgrMID_INF = _lgrMIDPrefix + "1101: ";
    protected static transient final String _lgrMID_WRN = _lgrMIDPrefix + "2001: ";
    protected static transient final String _lgrMID_ERR = _lgrMIDPrefix + "3001: ";
    protected static transient final String _lgrMID_EXC = _lgrMIDPrefix + "4001: ";

    static {                                                                        
        _loggerOC = Logger.getLogger(_lgrNameOutboundConnection);
        _loggerJF = Logger.getLogger(_lgrNameJMSConnectionFactory);
    }

    /** Constructor */
    public ConnectionFactoryAdapter(com.sun.messaging.jms.ra.ManagedConnectionFactory mcf,
        javax.resource.spi.ConnectionManager cm)
    {
        Object params[] = new Object[2];
        params[0] = mcf;
        params[1] = cm;

        _loggerOC.entering(_className, "constructor()", params);

        this.mcf = mcf;
        this.cm = cm;
    } 


    // Methods implementing javax.naming.Referenceable
    //
    public void setReference(Reference ref)
    {
        this.reference = ref;
    }
 
    public Reference getReference()
    throws NamingException
    {
        //MQJMSRA doesn't create or accept Reference objects (yet); throw the correct Exception
        throw new NamingException("MQRA:CFA:getReference:NOT Supported");
    }


    // Methods that implement javax.jms.ConnectionFactory //
    /**
     * Creates a Connection with the default user identity. The default user identity
     * is defined by the <code>ConnectionFactory</code> properties
     * <code><b>imqDefaultUsername</b></code> and <code><b>imqDefaultPassword</b></code>
     *   
     * @return a newly created Connection.
     *   
     * @exception JMSException if a JMS error occurs.
     * @see com.sun.messaging.ConnectionConfiguration#imqDefaultUsername
     * @see com.sun.messaging.ConnectionConfiguration#imqDefaultPassword
     */  
    public Connection
    createConnection()
    throws JMSException
    {
        _loggerJF.entering(_className, "createConnection()");
        //return createConnection(mcf.getUserName(), mcf.getPassword());
        return createConnection(null, null);
    }

    /**
     * Creates a Connection with a specified user identity.
     *   
     * @param username the caller's user name
     * @param password the caller's password
     *   
     * @return a newly created connection.
     *   
     * @exception JMSException if a JMS error occurs.
     */  
    public Connection
    createConnection(String username, String password)
    throws JMSException
    {
        _loggerJF.entering(_className, "createConnection()", username);
        return _createConnection(username, password);
    }

    private Connection
    _createConnection(String username, String password)
    throws JMSException
    {
        javax.resource.spi.ConnectionRequestInfo crinfo =
            new com.sun.messaging.jms.ra.ConnectionRequestInfo(mcf, username, password);

        //System.out.println("MQRA:CFA:createConnection:allocating connection");
        ConnectionAdapter ca;
        try {
            ca = (ConnectionAdapter)cm.allocateConnection(mcf, crinfo);
        } catch (ResourceException re) {
            //XXX:Fix codes
            String code = "1";
            throw new com.sun.messaging.jms.JMSException(
                "MQRA:CFA:allocation failure:createConnection:"+re.getMessage(), code, re);
        }
        return ca;
    }

    /**
     * Creates a Queue Connection with the default user identity. The default user identity
     * is defined by the <code>ConnectionFactory</code> properties
     * <code><b>imqDefaultUsername</b></code> and <code><b>imqDefaultPassword</b></code>
     *   
     * @return a newly created Queue Connection.
     *   
     * @exception JMSException if a JMS error occurs.
     * @see com.sun.messaging.ConnectionConfiguration#imqDefaultUsername
     * @see com.sun.messaging.ConnectionConfiguration#imqDefaultPassword
     */  
    public QueueConnection
    createQueueConnection()
    throws JMSException
    {
        _loggerJF.entering(_className, "createQueueConnection()");
        //return createQueueConnection(mcf.getUserName(), mcf.getPassword());
        return createQueueConnection(null, null);
    }

    /**
     * Creates a Queue Connection with a specified user identity.
     *   
     * @param username the caller's user name
     * @param password the caller's password
     *
     * @return a newly created queue connection.
     *
     * @exception JMSException if a JMS error occurs.
     */
    public QueueConnection
    createQueueConnection(String username, String password)
    throws JMSException
    {
        _loggerJF.entering(_className, "createQueueConnection()", username);
        return _createQueueConnection(username, password);
    }

    private QueueConnection
    _createQueueConnection(String username, String password)
    throws JMSException
    {
        javax.resource.spi.ConnectionRequestInfo crinfo =
            new com.sun.messaging.jms.ra.ConnectionRequestInfo(mcf, username, password);

        //System.out.println("MQRA:CFA:createQueueConnection:allocating Queue Connection");
        ConnectionAdapter ca;
        try {
            ca = (ConnectionAdapter)cm.allocateConnection(mcf, crinfo);
        } catch (ResourceException re) {
            //XXX:Fix codes
            String code = "1";
            throw new com.sun.messaging.jms.JMSException(
                "MQRA:CFA:allocation failure:createQueueConnection:"+re.getMessage(), code, re);
        }
        return (QueueConnection)ca;
    }

    /**
     * Creates a Topic Connection with the default user identity. The default user identity
     * is defined by the <code>ConnectionFactory</code> properties
     * <code><b>imqDefaultUsername</b></code> and <code><b>imqDefaultPassword</b></code>
     *   
     * @return a newly created Topic Connection.
     *   
     * @exception JMSException if a JMS error occurs.
     * @see com.sun.messaging.ConnectionConfiguration#imqDefaultUsername
     * @see com.sun.messaging.ConnectionConfiguration#imqDefaultPassword
     */  
    public TopicConnection
    createTopicConnection()
    throws JMSException
    {
        _loggerJF.entering(_className, "createTopicConnection()");
        //return createTopicConnection(mcf.getUserName(), mcf.getPassword());
        return createTopicConnection(null, null);
    }

    /**
     * Creates a Topic Connection with a specified user identity.
     *   
     * @param username the caller's user name
     * @param password the caller's password
     *
     * @return a newly created topic connection.
     *
     * @exception JMSException if a JMS error occurs.
     */
    public TopicConnection
    createTopicConnection(String username, String password)
    throws JMSException
    {
        _loggerJF.entering(_className, "createTopicConnection()", username);
        return _createTopicConnection(username, password);
    }

    private TopicConnection
    _createTopicConnection(String username, String password)
    throws JMSException
    {
        javax.resource.spi.ConnectionRequestInfo crinfo =
            new com.sun.messaging.jms.ra.ConnectionRequestInfo(mcf, username, password);

        //System.out.println("MQRA:CFA:createTopicConnection:allocating Topic Connection");
        ConnectionAdapter ca;
        try {
            ca = (ConnectionAdapter)cm.allocateConnection(mcf, crinfo);
        } catch (ResourceException re) {
            //XXX:Fix codes
            String code = "1";
            throw new com.sun.messaging.jms.JMSException(
                "MQRA:CFA:allocation failure:createTopicConnection:"+re.getMessage(), code, re);
        }
        return (TopicConnection)ca;
    }
}
