#!/sbin/sh
#
# @(#)imq-HP-UX.sh	1.4 05/08/15
#####hpux-dev#####
#
# Copyright 2005 Sun Microsystems, Inc. All Rights Reserved
# Use is subject to license terms.
#
# Permission to use, copy, modify, and distribute this software
# and its documentation for NON-COMMERCIAL purposes and without
# fee is hereby granted provided that this copyright notice
# appears in all copies. Please refer to the file "copyright.html"
# for further important copyright and licensing information.
#
# SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
# THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
# ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
# DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
#
######
#
# Sun Java(tm) System Message Queue Broker init.d script

#
# This script starts/stops the Message Queue broker process as part of 
# normal UNIX system operation based on the value of the
# AUTOSTART in the /etc/opt/sun/mq/imqbrokerd.conf file.
#
# To debug this script, set the environmental variable
# DEBUG to 1 (true)
#
# e.g.
# env DEBUG=1 /sbin/init.d/imq start
#
# This file is installed as:
#     /sbin/init.d/imq
#
# It is linked into the rc*.d directories as:
#	# ln imq /sbin/rc3.d/S52imq
#	# ln imq /sbin/rc0.d/K15imq
#	# ln imq /sbin/rc1.d/K15imq
#
# The S52imq link in /sbin/rc3.d will start the broker when
# the system enters multiuser mode. The K15imq links in
# rc0.d, rc1.d and rc2.d will bring down the broker when the
# system is brought down to single user, or administrative modes.
#
# The script looks at the file:  /etc/opt/sun/mq/imqbrokerd.conf to
# determine whether or not the broker should be started.

###### START LOCAL CONFIGURATION
# You may wish to modify these variables to suit your local configuration
######hpux-dev######
PATH=/sbin:/usr/sbin:/usr/bin
export PATH


# IMQ_HOME   Location of the Sun Java(tm) System Message Queue installation
IMQ_HOME=/opt/sun/mq
#
# Set this IMQ_HOME value if you relocated the sun-mq Depot
# to a location other than the default /opt/sun/mq
#
#IMQ_HOME=_IMQ_HOME_
export IMQ_HOME

# IMQ_VARHOME
# Set IMQ_VARHOME if you wish to designate an alternate
# location for storing broker specific data like:
#	- persistent storage
#	- SSL certificates
#	- user passwd database
#
# IMQ_VARHOME=/var/opt/sun/mq
# export IMQ_VARHOME

# BROKER_OPTIONS	Default Options to pass to the broker executable
# additional arguments can be defined in the ARGS property
#
BROKER_OPTIONS="-silent"

#
# ETC HOME
IMQ_ETCHOME="/etc/opt/sun/mq"
#
# imqbrokerd.conf
CONF_FILE=$IMQ_ETCHOME/imqbrokerd.conf
#
#
###### END LOCAL CONFIGURATION

EXECUTABLE=imqbrokerd


# error "description"
error () {
  echo $0: $* 2>&1
  exit 1
}

# find the named process(es)
findproc() {
  pid=`/bin/ps -ef |
  /bin/grep -w $1 | 
  /bin/grep -v grep |
  /bin/awk '{print $2}'`
  echo $pid
}

# kill the named process(es) (borrowed from S15nfs.server)
killproc() {
   pid=`findproc $1`
   [ "$pid" != "" ] && kill $pid
}

#
# require $IMQ_HOME
#

if [ ! -d "$IMQ_HOME" ]; then
   error "Cannot find Sun Java(tm) System Message Queue in IMQ_HOME ($IMQ_HOME)."
fi




#
# Start/stop Message Queue Broker
#
case "$1" in
'start')
         if [ -f $CONF_FILE ] ; then 
            AUTOSTART=`grep -v "^#" $CONF_FILE | grep AUTOSTART | cut -c11-`
            RESTART=`grep -v "^#" $CONF_FILE | grep RESTART | cut -c9-`
            ARGS=`grep -v "^#" $CONF_FILE | grep ARGS | cut -c6-`
            if [ $AUTOSTART ] ; then 
                if [ $AUTOSTART = "YES" -o $AUTOSTART = "Yes" -o $AUTOSTART = "yes" -o $AUTOSTART = "Y" -o $AUTOSTART = "y" ] ; then
                    START=1
                    REASON="The AUTOSTART property is turned on (set to $AUTOSTART) in  $CONF_FILE. The ARGS property is set to \"$ARGS\""

                elif [ $AUTOSTART = "NO" -o $AUTOSTART = "No" -o $AUTOSTART = "no" -o $AUTOSTART = "N" -o $AUTOSTART = "n" ] ; then
                    START=0
                    REASON="The AUTOSTART property is turned off (set to $AUTOSTART) in $CONF_FILE"
                else
                    START=0
                    REASON="The AUTOSTART property is not set to an expected value (value is $AUTOSTART)"
                fi
            else 
                START=0
                REASON="AUTOSTART was not set in $CONF_FILE"
            fi
         else
            START=0
            REASON="$CONF_FILE, the configuration file, was not found"
         fi

         # Handle restart option
         if [ ! -z "$RESTART" ]; then
             if [ $RESTART = "YES" ]; then
                BROKER_OPTIONS="-autorestart $BROKER_OPTIONS"
             fi
         fi

         if [ $START -eq 0 ] ; then
             if [ $DEBUG ] ; then
                 echo $IMQ_HOME/bin/$EXECUTABLE is not being started because [ $REASON ]
             fi
             exit 0
         else
             if [ $DEBUG ] ; then
                 echo $IMQ_HOME/bin/$EXECUTABLE is starting because [ $REASON ]
             fi
         fi

         if [ $DEBUG ] ; then 
	     echo "Command \c"
	     echo "$IMQ_HOME/bin/$EXECUTABLE -bgnd $BROKER_OPTIONS  $ARGS... \c"
         fi
	 cd $IMQ_HOME

	 # Check if the server is already running.
	 if [ -n "`findproc $EXECUTABLE`" ]; then
             if [ $DEBUG ] ; then 
	        echo "$EXECUTABLE is already running."
             fi
	     exit 0
	 fi

	 bin/$EXECUTABLE -bgnd $BROKER_OPTIONS $ARGS &
         if [ $DEBUG ] ; then 
  	     echo "done"
         fi

	;;

'stop')
        if [ $DEBUG ] ; then
	    echo -n "Stopping $EXECUTABLE ... "
        fi
	if [ -z "`findproc $EXECUTABLE`" ]; then
            if [ $DEBUG ] ; then
	        echo "$EXECUTABLE is not running."
            fi
	    exit 0
	fi
	killproc $EXECUTABLE
        if [ $DEBUG ] ; then
	    echo "done"
        fi
	;;

*)
	echo "Usage: $0 { start | stop }"
	;;
esac
